==> dasu_4.11.jar: edu/northwestern/dasu/dynamic/measurement/DasuNdtModule.java <==
/**
 * Dasu Project
 *
 * File:         DasuNdtModule.java
 * RCS:          $Id$
 * Description:  OptimalEdge class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 15, 2008 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.dynamic.measurement;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;

import edu.northwestern.cs.aqualab.Cattle.json.JSONObject;
import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactTemplate;
import edu.northwestern.dasu.dynamic.drools.FactNdtProbeResult;
import edu.northwestern.dasu.measurement.DasuProbeModule;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.netstat.NetstatBWStats;
import edu.northwestern.dasu.netstat.NetstatBanwidthPerformer;
import edu.northwestern.dasu.util.Pair;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The DasuNdtModule class manages the set of 
 *         NDT probes performed by the peer.
 */

public class DasuNdtModule extends DasuProbeModule<FactNdtProbeResult> {	
	
	public Hashtable<Long, FactNdtProbeResult> ndtResultTable;
	private ArrayList<ProbeSubTask> toDoServerList = new ArrayList<ProbeSubTask>();
	private static DasuNdtModule self;
	protected static Tcpbw100 ndtTest = new Tcpbw100();
	private static LinkedList<String> servers = new LinkedList<String>();
	
	public DasuNdtModule() {
				
		super("DasuNdtModule", 1.0f);
		this.pi = Main.getPluginInterface();
		ndtResultTable = new Hashtable<Long, FactNdtProbeResult>();
		
		active = true;
	}
	
	public void loadServerList() {
		
		if ( servers.size() > 0 )
			return;
		
		JSONObject json = null;
		try {
			json = DasuManager.getInstance().getConfigService().GETData( DasuConfiguration.DASU_SCRIPT_NDT + 
					DasuConfiguration.getInstance().getResourceScriptRequest() );
		}
		catch (Exception e) {
			//ignore
		}
			
		if ( json != null ) {
			
			synchronized(servers) {
				
				Iterator<String> iter = json.keys();
				while ( iter.hasNext() ) {					
					String key = iter.next();
					
					try {
						servers.add( json.getString(key) );
					}
					catch (Exception e) {
						LOGGER.info("NDT: " + e.getMessage());
					}
				}
				
				LOGGER.info("NDT: " + servers);
			}
		}
		
	}
	
	public synchronized static DasuNdtModule getInstance() {
		if (self == null) {
			self = new DasuNdtModule();
			self.setDaemon(true);
		}
		return self;
		
	}

	@Override
	public void run() {
		
		synchronized(servers) {
			loadServerList();
		}
		
		while (!Main.isShuttingDown() && isActive()) {
		 
			ProbeSubTask pst = null;
			synchronized (toDoServerList) {
				while ((toDoServerList.size()==0) && !Main.isShuttingDown()) {
					try {
						toDoServerList.wait(sleepInterval*1000);
					} catch (InterruptedException e) {
						setActive(false);
						LOGGER.info("STOP: Adios...");
						return;
					}
				}
				
				if ( toDoServerList.size() > 0)
					pst = toDoServerList.remove(0);
			}
			
			if ( pst != null ) {
			
				while (isPause() && isActive()) {
					try {
						Thread.sleep(pauseInterval*1000);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				
				Object ret = probeIp(pst);
				if ( ret != null ) {
					
					FactNdtProbeResult result = (FactNdtProbeResult) ret;
					result.setPst(pst);
					
					updateNdtResultQueue(result);
					
					if ( yieldInterval != -1) {
						try {
							sleep(yieldInterval);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				}
			}


		} // end while active

		LOGGER.info("STOP: Adios...");
	}
		
	@Override
	public void probeAll(ProbeTask pt) {
		
		if ( pt != null) {
			
			synchronized (toDoServerList) {
				for(ProbeSubTask pst: pt.getSubTasks()) {
					if (pst.ipAddress == null) {
						pst.ipAddress = "";
					}
					toDoServerList.add(pst);
				}
				toDoServerList.notifyAll();
			}
		}
	}
	
	@Override
	public void probeWithPriority(ProbeSubTask pst) {
		
		if (pst.ipAddress == null) {			
			pst.ipAddress = "";
		}
			
		synchronized (toDoServerList) {
			toDoServerList.add(pst);
			toDoServerList.notifyAll();
		}		
	}

	@Override
	public void probe(ProbeSubTask pst) {

		if ( pst != null) {
			
			if (pst.ipAddress == null)			
				pst.ipAddress = "";
	
			synchronized (toDoServerList) {
				toDoServerList.add(pst);
				toDoServerList.notifyAll();
			}
		}
	}
	
	protected String getServerFromList() {
	
		synchronized(servers) {
			Random rand = new Random();
	
			if (servers.size() == 0)
				loadServerList();
			
			if (servers.size() == 0) {
				return "ndt.iupui.donar.measurement-lab.org";
			}
			
			return servers.get(rand.nextInt( servers.size()));
		}
		

	}
	
	@Override
	protected Object probeIp(ProbeSubTask pst) {
	
		if ( !isActive() )
			return null;
		
		String origAddress = pst.ipAddress;
		if (pst.ipAddress.equals("")) {
			pst.ipAddress = getServerFromList();
		}
		
		FactNdtProbeResult ret = new FactNdtProbeResult(pst.parent.probeType);
		
		ret.setTarget(pst.target);	
		ret.setPst(pst);
		ret.requestedServer = origAddress;

		NetstatBanwidthPerformer nbp = new NetstatBanwidthPerformer(Main.getInterfaceName(), "ndt netstat");
		nbp.loop(1000);
		
    	Long timeBefore = NetstatBWStats.getInstance().getMostRecent().timeRecorded;
    	LOGGER.info("NDT: NETSTAT BEFORE: " + NetstatBWStats.getInstance());
    	
	    try {
	    	
	    	HashMap<String, Object> map = ndtTest.dottcp(pst.ipAddress);
	    	
	    	for ( String key : map.keySet() ) {
	    		ret.getClass().getField(key).set(ret, map.get(key));
	    	}
	    	
	    	ret.interfaceName = Main.getInterfaceName();
	    	
	    	/*
	    	 * sleep for five seconds after ndt finishes
	    	 * to ensure timer already fired at least once
	    	 * before trying to destroy it 
	    	 * in case ndt measurement fails
	    	 */
	    	
	    	Thread.sleep(5000);

		} catch (IOException e) {
			LOGGER.info("NDT: ERROR: " + e.getMessage());
			//e.printStackTrace();
			
	    	try {
				Thread.sleep(5000);
			} catch (InterruptedException e1) {
				//e1.printStackTrace();
				LOGGER.info("NDT: ERROR: " + e1.getMessage());
			}
	    	
			
	    	ret.failed = true;
			    	
		} catch (Exception e) {
			LOGGER.info("NDT: ERROR: " + e.getMessage());
			ret.failed = true;
		}
		
		nbp.cancelTimer();
		
		Long timeAfter = NetstatBWStats.getInstance().getMostRecent().timeRecorded;
    	
		ret.c2sRateNetstat = (double) NetstatBWStats.getInstance().getPercentileUploadBetween(timeBefore, timeAfter, 95)/1000;
		ret.s2cRateNetstat = (double) NetstatBWStats.getInstance().getPercentileDownloadBetween(timeBefore, timeAfter, 95)/1000;

		reportProbeResult(ret);
		//Statistics.getInstance().addNdtResultDasu(ret);

		StringBuilder sb = new StringBuilder();
		sb.append("NDT: RESULT: timeBefore: " + timeBefore + " timeAfter: " + timeAfter + "\n");
		sb.append("NDT: ndt received:" + ret.getS2cRate() + "\n");
		sb.append("NDT: Netstat 95th received:" + ret.getS2cRateNetstat()+ "\n");
		sb.append("NDT: Netstat max received: " + NetstatBWStats.getInstance().getFastestDownloadBetween(timeBefore, timeAfter)/1000+ "\n");
		sb.append("NDT: ndt sent:" + ret.getC2sRate()+ "\n");
		sb.append("NDT: Netstat 95th sent:" + ret.getC2sRateNetstat()+ "\n");
		sb.append("NDT: Netstat max sent: " + NetstatBWStats.getInstance().getFastestUploadBetween(timeBefore, timeAfter)/1000+ "\n");
		sb.append("NDT: message: " + ret.errorMsg+ "\n");
		sb.append("NDT: NETSTAT AFTER: " + NetstatBWStats.getInstance()+ "\n");
		
		LOGGER.info(sb.toString());
		
		if (ret.getErrorMsg() != null)
			ret.failed = true;
		
		return ret;
		
	}
	
	private void updateNdtResultQueue(FactNdtProbeResult result) {

		Long id = generateId();
		ndtResultTable.put(id, result);
		updateResultQueue(result, result.getProbeType(), id);
						
		synchronized(servers) {

			if ( ((result.failed) && servers.contains(result.server)) ) {
					servers.remove(result.server);
			}
		}
	}


	public static LinkedList<String> getServers() {
		return servers;
	}
		
	
	@Override
	public void stopModule() {
		
		LOGGER.info("STOP: stopping DasuNdtModule...");
		setActive(false);
		synchronized(toDoServerList) {			
			toDoServerList.notify();
		}		
	}

	public Pair<String, FactTemplate> proccessResult(long eventId) {
		
		FactNdtProbeResult ndtResult = ndtResultTable.remove(eventId);
		return new Pair<String, FactTemplate>(ndtResult.requestedServer, ndtResult);
		
	}

}

==> dasu_5.31.jar: edu/northwestern/dasu/dynamic/measurement/DasuNdtModule.java <==
/**
 * Dasu Project
 *
 * File:         DasuNdtModule.java
 * RCS:          $Id$
 * Description:  DasuNdtModule class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 18, 2013 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.dynamic.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.dynamic.measurement;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Random;

import edu.northwestern.cs.aqualab.cattle.jsonsimple.JSONObject;
import edu.northwestern.cs.aqualab.cattle.jsonsimple.parser.JSONParser;
import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.dynamic.drools.FactNdtProbeResult;
import edu.northwestern.dasu.measurement.DasuProbeModule;
import edu.northwestern.dasu.measurement.tasks.ProbeTask;
import edu.northwestern.dasu.measurement.tasks.ProbeTask.ProbeSubTask;
import edu.northwestern.dasu.netstat.NetstatBWStats;
import edu.northwestern.dasu.netstat.NetstatBanwidthPerformer;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The DasuNdtModule class manages the set of 
 *         NDT probes performed by the client.
 */

public class DasuNdtModule extends DasuProbeModule<FactNdtProbeResult> {	
	
	private static DasuNdtModule self;
	protected static Tcpbw100 ndtTest = new Tcpbw100();
	private static LinkedList<String> servers = new LinkedList<String>();
	private static JSONObject donar;
	
	public DasuNdtModule() {
		super("DasuNdtModule", 3.0f);
		donar = new JSONObject();
		loadServerInfoThroughDonar();
		synchronized(servers) {
			loadServerList();			
		}
	}
	
	public boolean isIdle() {
		return idle;
	}
	
	private void loadServerInfoThroughDonar() {
		
		synchronized(donar) {
			donar = GETJSONData("http://mlab-ns.appspot.com", 80, "ndt?format=json");
		}
	}
	
	public void loadServerList() {
		
		if ( servers.size() > 0 )
			return;
		
		JSONObject json = null;
		try {
			json = DasuManager.getInstance().getConfigService().GETData( DasuConfiguration.DASU_SCRIPT_NDT + 
					DasuConfiguration.getInstance().getResourceScriptRequest() );
		}
		catch (Exception e) {
			//ignore
		}
			
		if ( json != null ) {
			
			synchronized(servers) {
				
				@SuppressWarnings("unchecked")
				Iterator<String> iter = json.keySet().iterator();
				while ( iter.hasNext() ) {					
					String key = iter.next();
					
					try {
						servers.add( (String) json.get(key) );
					}
					catch (Exception e) {
						LOGGER.info("NDT: " + e.getMessage());
					}
				}
				
				LOGGER.info("NDT: " + servers);
			}
		}
		
	}
	
	public synchronized static DasuNdtModule getInstance() {
		if (self == null) {
			self = new DasuNdtModule();
			self.setDaemon(true);
		}
		return self;
		
	}
		
	@Override
	public void probeAll(ProbeTask pt) {
		super.probeAll(pt);
		if ( pt != null) {
			
			synchronized (toDoTaskList) {
				for(ProbeSubTask pst: pt.getSubTasks()) {
					if (pst.getToProbe() == null) {
						pst.setToProbe("");
					}
					toDoTaskList.add(pst);
				}
				toDoTaskList.notifyAll();
			}
		}
	}
	
	@Override
	public void probeWithPriority(ProbeSubTask pst) {
		
		if (pst.getToProbe() == null) {			
			pst.setToProbe("");
		}
		
		super.probeWithPriority(pst);
		
	}

	@Override
	public void probe(ProbeSubTask pst) {

		if ( pst != null) {
			
			if (pst.getToProbe() == null)			
				pst.setToProbe("");
	
			super.probe(pst);
		}
	}
	
	protected String getServerFromList() {
	
		synchronized(donar) {
			
			//try again
			if ( !donar.containsKey("fqdn") )
				loadServerInfoThroughDonar();
		}
		
		if ( donar.containsKey("fqdn") )
			return donar.get("fqdn").toString();			
		else {	
		
			synchronized(servers) {
				Random rand = new Random();
		
				if (servers.size() == 0)
					loadServerList();
				
				if (servers.size() == 0) {
					return "ndt.iupui.donar.measurement-lab.org";
				}
				
				return servers.get(rand.nextInt( servers.size()));
			}
		}
			
	}
		
	protected FactNdtProbeResult probeIp(ProbeSubTask pst) {
	
		if ( !isActive() )
			return null;
		
		String origAddress = pst.getToProbe();
		if (pst.getToProbe().equals("")) {
			pst.setToProbe( getServerFromList() );
		}
		
		FactNdtProbeResult ret = new FactNdtProbeResult(pst.getParent().probeType);
		
		ret.setTarget(pst.getTarget());	
		ret.setPst(pst);
		ret.setRequestedServer( origAddress );

		NetstatBanwidthPerformer nbp = new NetstatBanwidthPerformer(Main.getInterfaceName(), "ndt netstat");
		nbp.loop(1000);
		
    	Long timeBefore = NetstatBWStats.getInstance().getMostRecent().timeRecorded;
    	LOGGER.info("NDT: NETSTAT BEFORE: " + NetstatBWStats.getInstance());
    	
	    try {
	    	
	    	HashMap<String, Object> map = ndtTest.dottcp(pst.getToProbe());
	    	
	    	for ( String key : map.keySet() ) {
	    		ret.getClass().getField(key).set(ret, map.get(key));
	    	}
	    	
	    	ret.interfaceName = Main.getInterfaceName();
	    	
	    	/*
	    	 * sleep for five seconds after ndt finishes
	    	 * to ensure timer already fired at least once
	    	 * before trying to destroy it 
	    	 * in case ndt measurement fails
	    	 */
	    	
	    	Thread.sleep(5000);

		} catch (IOException e) {
			LOGGER.info("NDT: ERROR: " + e.getMessage());
			
	    	try {
				Thread.sleep(5000);
			} catch (InterruptedException e1) {
				LOGGER.info("NDT: ERROR: " + e1.getMessage());
			}
	    	
			
	    	ret.setFailed(true);
			    	
		} catch (Exception e) {
			LOGGER.info("NDT: ERROR: " + e.getMessage());
			ret.setFailed(true);
		}
		
		nbp.cancelTimer();
		
		Long timeAfter = NetstatBWStats.getInstance().getMostRecent().timeRecorded;
    	
		ret.setC2sRateNetstat( (double) NetstatBWStats.getInstance().getPercentileUploadBetween(timeBefore, timeAfter, 95)/1000 );
		ret.setS2cRateNetstat( (double) NetstatBWStats.getInstance().getPercentileDownloadBetween(timeBefore, timeAfter, 95)/1000 );

		StringBuilder sb = new StringBuilder();
		sb.append("NDT: RESULT: timeBefore: " + timeBefore + " timeAfter: " + timeAfter + "\n");
		sb.append("NDT: ndt received:" + ret.getS2cRate() + "\n");
		sb.append("NDT: Netstat 95th received:" + ret.getS2cRateNetstat()+ "\n");
		sb.append("NDT: Netstat max received: " + NetstatBWStats.getInstance().getFastestDownloadBetween(timeBefore, timeAfter)/1000+ "\n");
		sb.append("NDT: ndt sent:" + ret.getC2sRate()+ "\n");
		sb.append("NDT: Netstat 95th sent:" + ret.getC2sRateNetstat()+ "\n");
		sb.append("NDT: Netstat max sent: " + NetstatBWStats.getInstance().getFastestUploadBetween(timeBefore, timeAfter)/1000+ "\n");
		sb.append("NDT: message: " + ret.errorMsg+ "\n");
		sb.append("NDT: NETSTAT AFTER: " + NetstatBWStats.getInstance()+ "\n");
		
		LOGGER.info(sb.toString());
		
		if (ret.getErrorMsg() != null)
			ret.setFailed(true);
		
		if ( donar.containsKey("fqdn") ) 
			ret.setServerSelectedThroughDonar(true);
		
		reportProbeResult(ret);
		return ret;
		
	}
		
	public void afterUpdateResultQueue(FactNdtProbeResult result) {
						
		synchronized(servers) {

			if ( ((result.failed) && servers.contains(result.server)) ) {
					servers.remove(result.server);
			}
		}
	}

	public static LinkedList<String> getServers() {
		return servers;
	}
		
    private JSONObject GETJSONData(String host, Integer port, String resource) {
    	
        JSONObject ret = null; 
        
    	try {
    		
    		LOGGER.info("NDT: Contacting " + host + ":" + port + "/" + resource);
    		
    		URL url = new URL(host + ":" + port + "/" + resource);
	    	HttpURLConnection conn = (HttpURLConnection)url.openConnection();

	    	InputStream inputstream = conn.getInputStream();
	    	InputStreamReader inputstreamreader = new InputStreamReader(inputstream);
	    	BufferedReader bufferedreader = new BufferedReader(inputstreamreader);
	
	    	String string = null;
	    	String res = new String();
	    	while ((string = bufferedreader.readLine()) != null) {	    	    
	    	    res = res.concat(string);
	    	}
	    	
	    	res = res.replace("'", "\"");
	    	LOGGER.info("NDT: Received (" + res + ")");
	        ret = (JSONObject)new JSONParser().parse(res);
	    	
    	}
    	catch (Exception e) {
    		LOGGER.info("NDT: ERROR: " + e.getMessage());
    	}
    	
    	return ret;
    }	

	@Override
	public Integer getNumberActiveProbes() {
		
		if (!isIdle())
			return 1;
		
		return 0;
		
	}

    
}
